@@ -11,18 +11,18 @@ use warnings;
 use warnings::register;
 
 use vars qw($VERSION $DATE $FILE);
-$VERSION = '0.01';
-$DATE = '2003/08/04';
+$VERSION = '0.02';
+$DATE = '2003/09/12';
 $FILE = __FILE__;
 
 use vars qw(@ISA @EXPORT_OK);
 require Exporter;
 use Archive::Tar;
 @ISA= qw(Exporter Archive::Tar);
-@EXPORT_OK = qw(tar untar);
-
-# use SelfLoader;
+@EXPORT_OK = qw(tar untar parse_header);
 
+use SelfLoader;
+use Tie::Gzip;
 use Cwd;
 
 ######
@@ -30,34 +30,51 @@ use Cwd;
 #
 sub new
 {
-     my $class = shift @_;
+
+     ####################
+     # $class is either a package name (scalar) or
+     # an object with a data pointer and a reference
+     # to a package name. A package name is also the
+     # name of a class
+     #
+     my ($class, @args) = @_;
      $class = ref($class) if( ref($class) );
 
      ######
-     # Parse the arguments
+     # Parse the last argument as options if it is a reference.
      #
-     my ($tar_file, $compress, $args_p);
-     if( ref($_[-1]) eq 'ARRAY') {
-          $args_p = $_[-1];
-          my %args = @{$args_p};
-          $args_p = \%args;
-     }
-     elsif (ref($_[-1]) eq 'HASH') {
-          $args_p = pop @_;
+     my $options = {};
+     my ($tar_file, $compress);
+     if( ref($args[-1]) ) {
+          $options = pop @args;
+          if(ref($options) eq 'ARRAY') {
+              my %options = @{$options};
+              $options = \%options;
+          }
+          ($tar_file, $compress) = @args;
+          
      }
      else {
-          $tar_file = shift @_;
-          $compress = shift @_;
-          my %args = @_;
-          $args_p = \%args;
+          ($tar_file, $compress, my @options) = @args;
+          my %options = @options;
+          $options = \%options;
      }
 
      #######
-     # tar object
+     # Using the Archive::Tar class and putting layer around it.
+     # Bypassing the compress in Archive::Tar and using the 
+     # compress herein. In this way if the Compress::Zlib package
+     # is not present, the class methods herein will try to fall
+     # back onto the gzip operating system common. A lot of Unix
+     # ISP do not install Compress:Zlib because the availability of
+     # the gzip command.
      #
-     my $self = $class->Archive::Tar::new($tar_file, $compress);
-     $self->{TarGzip} = $args_p;
+     my $self = $class->Archive::Tar::new($tar_file, 0, $options);
+     $options->{tar_file} = $tar_file if defined $tar_file;
+     $options->{compress} = $compress if defined $compress;
+     $self->{TarGzipDefault} = $options;
      $self;
+
 }
 
 
@@ -66,120 +83,135 @@ sub new
 #
 sub taropen 
 {
-     my $self = shift @_;
+     my ($self,@args) = @_;
 
      ######
      # Parse the arguments
      #
-     my ($tar_file, $compress, $args_p);
-     if( ref($_[-1]) eq 'ARRAY') {
-          $args_p = $_[-1];
-          my %args = @{$args_p};
-          $args_p = \%args;
-     }
-     elsif (ref($_[-1]) eq 'HASH') {
-          $args_p = pop @_;
+     ######
+     # Parse the last argument as options if it is a reference.
+     #
+     my $options = {};
+     my ($tar_file, $compress);
+     if( ref($args[-1]) ) {
+          $options = pop @_;
+          if(ref($options) eq 'ARRAY') {
+              my %options = @{$options};
+              $options = \%options;
+          }
+          ($tar_file, $compress) = @args;
+          
      }
      else {
-          $tar_file = shift @_;
-          $compress = shift @_;
-          my %args = @_;
-          $args_p = \%args;
+          ($tar_file, $compress, my @options) = @args;
+          my %options = @options;
+          $options = \%options;
      }
 
-     my $key;
-     foreach $key (%$args_p) {
-         $self->{TarGzip}->{$key} = $args_p->{$key};
+     ###############
+     # The options in taropen method override any default options 
+     # establihed by the new method
+     #
+     my %options = %{$self->{TarGzipDefault}};
+     foreach my $key (%$options) {
+         $options{$key} = $options->{$key};
      }
+  
+     $options{compress} = $compress if defined $compress;
+     $options{tar_file} = $tar_file if defined $tar_file;
+     $options{gz_suffix} = '.gz' unless $options{gz_suffix};
+     $options{tar_suffix} = '.tar' unless $options{tar_suffix};
+     $self->{TarGzip}  = \%options;
 
-     $self->{TarGzip}->{compress} = $compress if $compress;
-
-     $tar_file = $self->{TarGzip}->{tar_file} unless $tar_file;
-     unless( $tar_file ) {
+     unless(  defined $options{tar_file} ) {
          warn( "No tar file\n");
          return undef;
      }
-     $self->{TarGzip}->{tar_file} = $tar_file;
 
      #######
      # Try to find a file by adding extension for the tar and gz
      #
+     ($tar_file,$compress) = ($options{tar_file},$options{compress});
+
+     my ($tar_suffix,$gz_suffix) = ($options{tar_suffix},$options{gz_suffix});
+     my $targz_suffix = $tar_suffix . $gz_suffix;  
+     my $tar_length = length($tar_suffix);
+     my $targz_length = length($targz_suffix);
+
      my $flag = $self->{TarGzip}->{tar_flag};
+
+     #########################
+     # Always write to a file using the correct suffices. TarGzip follows very
+     # strict rules on what it writes out.
+     #
      if( $flag eq '>' ) {
-         if( $self->{TarGzip}->{compress} ) {
-             $tar_file .= '.gz' if substr($tar_file, -4, 4) eq '.tar';
-             $tar_file .= '.tar.gz' unless substr($tar_file, -7, 7) eq '.tar.gz';
+         if( $compress ) {
+             $tar_file .= $gz_suffix if substr($tar_file, -$tar_length, $tar_length) eq $tar_suffix;
+             $tar_file .= $targz_suffix unless substr($tar_file, -$targz_length, $targz_length) eq $targz_suffix;
          }
          else {
-             $tar_file .= '.tar' unless substr($tar_file, -4, 4) eq '.tar';
+             $tar_file .= $tar_suffix unless substr($tar_file, -$tar_length, $tar_length) eq $tar_suffix;
          }
      }
+
+     ################
+     # TarGzip is lenient on the file extensions it accepts for reading.
+     #
      else {
          unless( -e $tar_file ) {
-             if( $self->{TarGzip}->{compress} ) {
-                 $tar_file .= '.gz' if substr($tar_file, -4, 4) eq '.tar';
-                 $tar_file .= '.tar.gz' unless substr($tar_file, -7, 7) eq '.tar.gz';
+             if( $compress ) {
+                 $tar_file .= $gz_suffix if substr($tar_file, -$tar_length, $tar_length) eq $tar_suffix;
+                 $tar_file .= $targz_suffix unless substr($tar_file, -$targz_length, $targz_length) eq $targz_suffix;
              }
              else {
-                 $tar_file .= '.tar' unless substr($tar_file, -4, 4) eq '.tar';
+                 $tar_file .= $tar_suffix unless substr($tar_file, -$tar_length, $tar_length) eq $tar_suffix;
              }
-             $self->{TarGzip}->{tar_file} = $tar_file;
              unless( -e $tar_file ) {
                  warn("Cannot find $tar_file\n");
                  return undef;
              }
          }
      }
+     $options{tar_file} = $tar_file;
 
      ########
-     # Open files
+     # Use special tie to gzip if compressing
+     # the tar file.
      # 
-     if( $self->{TarGzip}->{compress} ) {
-         require Compress::Zlib;
-         Compress::Zlib->import qw(&gzopen $gzerrno Z_STREAM_END);
-         $self->{TarGzip}->{handle} = gzopen($tar_file, ($flag eq '>' ) ? 'wb' : 'rb');
-     }
-     else {
+     tie *TAR, 'Tie::Gzip' if $compress;
 
-         ######
-         # Open tar file
-         #
-         unless (open TAR, "$flag $tar_file") {
-             warn( "Cannot open $flag $tar_file\n");
-             return undef;
-         }
-         binmode TAR;
-         $self->{TarGzip}->{handle} = \*TAR;
+     ######
+     # Open tar file
+     #
+     unless (open TAR, "$flag $tar_file") {
+         warn( "Cannot open $flag $tar_file\n");
+         return undef;
      }
+     binmode TAR;
+     $self->{TarGzip}->{handle} = \*TAR;
      $self->{TarGzip}->{handle};
+
 }
 
+
+
 #######
 # Close the TAR file
 #
 sub tarclose
 {
-
      my ($self) = @_;
      my $handle = $self->{TarGzip}->{handle};
      return undef unless $handle;
-     my $blank_header = "\0" x 1024;  
-     my $write_flag = $self->{TarGzip}->{tar_flag} eq '>';
-     $self->{TarGzip}->{handle} = '';
-     if( $self->{TarGzip}->{compress} ) {
-          $handle->gzwrite($blank_header) if $write_flag;
-          $handle->gzclose( );
-     }
-     else {
-          print $blank_header if $write_flag; 
-          close $handle;
-     }
-
+     print $handle "\0" x 1024 if $self->{TarGzip}->{tar_flag} eq '>'; 
+     my $success = close $handle;
+     $self->{TarGzip}->{handle} = undef;
+     $success;
 }
 
-# 1
+1
 
-# __DATA__
+__DATA__
 
 
 #######
@@ -197,8 +229,23 @@ sub taradd
          binmode FILE;
          $file_contents = join '', <FILE>;
          close FILE;
+ 
+         ############################
+         # Do not add empty files to tar archive file
+         #
+         return 1 unless $file_contents;
+
      }
      return undef unless $self->add_data($file_name, $file_contents);
+
+     #####################
+     # Set the mode to 0777 for directories so chdir will work;
+     # otherwise, a regular file and set the mode to 0666
+     #
+     $self->{_data}->[0]->{mode} = $file_contents ? 0666 : 0777;
+     my $umask = $self->{TarGzip}->{umask};
+     $umask = umask unless $umask;
+     $self->{_data}->[0]->{mode} &= (0777 - $umask); 
      $self->tarwrite();
 
      1
@@ -213,35 +260,27 @@ sub tarwrite
      my ($self) = @_;
      my $tar_content = &Archive::Tar::format_tar_entry(shift @{$self->{'_data'}});
      my $handle = $self->{TarGzip}->{handle};
-     if( $self->{TarGzip}->{compress} ) {
-          my $gz = $handle;
-          unless ($gz->gzwrite($tar_content) ) {
-              warn "error gzwrite $self->{TarGzip}->{tar_file}.gz: $Compress::Zlib::gzerrno\n" ;
-              $gz->gzclose( );
-              return undef;
-          }
+     my $bytes_written;
+     unless($bytes_written = print $handle $tar_content ) {
+          warn "print error\n" ;
+          close $self->{TarGzip}->{handle};
+          return undef;
      }
-     else {
-          unless( print $handle $tar_content ) {
-              warn "print error\n" ;
-              close $self->{TarGzip}->{handle};
-              return undef;
-          }
-     }
-
-     1
+     $bytes_written;
 }
 
 
 
 ######
 # This is taken directly from big loop in Archive::Tar::read_tar
-#
+# Need to get it out of the loop for use in this module
 #
 sub parse_header
 {
-     return undef unless(defined($_[0]));      
-     shift @_ if $_[0] eq 'Archive::TarGzip' || ref($_[0]);  # drop self on object call 
+     ######
+     # This subroutine uses no object data.
+     #
+     shift @_ if UNIVERSAL::isa($_[0],__PACKAGE__);
 
      unless(@_) {
          warn "No arguments.\n";
@@ -250,6 +289,7 @@ sub parse_header
 
      my ($header) = @_;
 
+     ########
      # Apparently this should really be two blocks of 512 zeroes,
      # but GNU tar sometimes gets it wrong. See comment in the
      # source code (tar.c) to GNU cpio.
@@ -282,6 +322,8 @@ sub parse_header
      $devminor = oct $devminor;
      $name = $prefix."/".$name if $prefix;
      $prefix = "";
+ 
+     #########
      # some broken tar-s don't set the typeflag for directories
      # so we ass_u_me a directory if the name ends in slash
      $typeflag = 5 if $name =~ m|/$| and not $typeflag;
@@ -333,18 +375,18 @@ sub tarread
      #######
      # Parse out any options
      #
-     my $options_p;
+     my $options;
      if( ref($_[-1]) eq 'ARRAY') {
-          $options_p = $_[-1];
-          my %options = @{$options_p};
-          $options_p = \%options;
+          $options = $_[-1];
+          my %options = @{$options};
+          $options = \%options;
      }
      elsif (ref($_[-1]) eq 'HASH') {
-          $options_p = pop @_;
+          $options = pop @_;
      }
 
-     foreach my $key (keys %$options_p) {
-         $self->{TarGzip}->{$key} = $options_p->{$key}; 
+     foreach my $key (keys %$options) {
+         $self->{TarGzip}->{$key} = $options->{$key}; 
      }
 
      ####### 
@@ -364,7 +406,7 @@ sub tarread
      # Parse header
      #
      my $file_position = undef;
-     $file_position = tell $self->{TarGzip}->{handle} unless( $self->{TarGzip}->{compress} );
+     $file_position = tell $self->{TarGzip}->{handle};
      my $header = parse_header( $data );
      return undef if $header->{end_of_tar};
      $header->{file_position} = $file_position if $file_position;
@@ -429,67 +471,45 @@ sub target
      my $handle = $self->{TarGzip}->{handle};
      $handle = $self->{TarGzip}->{handle};
      my $bytes;
-     if( $self->{TarGzip}->{compress} ) {
-         my $trash;
-         unless( $buffer_p ) {
-             $buffer_p = \$trash;
-         }
-         $$buffer_p = '';
-         $bytes = $handle->gzread($$buffer_p, $size);
-         if( $bytes == -1) {
-              warn "error gzwrite $self->{TarGzip}->{tar_file}\.gz: $Archive::Zlib::gzerrno\n" ;
-              $handle->gzclose( );
-              return undef;
-         }
-         return undef unless $bytes == $size || !$buffer_p;
-     }
-     elsif( $buffer_p ) {
+     if( $buffer_p ) {
          $$buffer_p = '';
          $bytes = read( $handle, $$buffer_p, $size);
          return undef unless $bytes == $size;
      }
      else {
          seek $handle, $size, 1;
+         $bytes = $size;
      }
-
-     1
+     $size
 }
 
 
 
 
-
 #######
 # Store a number of files in one archive file in the tar format
 #
 sub tar
 {
-     unless(defined($_[0])) {
-         warn( "No inputs.\n");
-         return undef;
-     }
-     shift @_ if $_[0] eq 'Archive::TarGzip' || ref($_[0]);  # drop self on object call 
-
-     my (@files) = @_;
-
-     unless(@files) {
-         warn "No files.\n";
-         return undef;
-     }
+     ######
+     # This subroutine uses no object data.
+     #
+     shift @_ if UNIVERSAL::isa($_[0],__PACKAGE__);
 
      ######
-     # Check to see if first argument is an option
+     # pop last argument if last argument is an option
      #
-     my $options_p;
-     if( ref($files[-1]) eq 'ARRAY') {
-          my %options = @{pop @files};
-          $options_p = \%options;
-     }
-     elsif (ref($files[-1]) eq 'HASH') {
-          $options_p = pop @files;
+     my $options = pop @_ if ref($_[-1]);
+     if( ref($options) eq 'ARRAY') {
+          my %options = @{$options};
+          $options = \%options;
      }
 
-     unless(@files) {
+     ##############
+     # Rest of the arguments are file names and must
+     # have at least one file name
+     #
+     unless(@_) {
          warn "No files.\n";
          return undef;
      }
@@ -497,9 +517,9 @@ sub tar
      ########
      # Create a new tar file
      #
-     $options_p->{tar_flag} = '>';
+     $options->{tar_flag} = '>';
      my $tar;  
-     return undef unless $tar = new Archive::TarGzip($options_p);
+     return undef unless $tar = new Archive::TarGzip($options);
      return undef unless $tar->taropen();
 
      #####
@@ -509,22 +529,25 @@ sub tar
      require File::AnySpec;
 
      ######
-     # Process options
+     # Process dest_dir and src_dir options
      #
-     $options_p->{dest_dir} = '' unless $options_p->{dest_dir};
-     my $dest_dir = $options_p->{dest_dir};
+     $options->{dest_dir} = '' unless $options->{dest_dir};
+     my $dest_dir = $options->{dest_dir};
      $dest_dir = File::AnySpec->os2fspec('Unix',$dest_dir,'nofile');
      $dest_dir .= '/' unless $dest_dir && substr($dest_dir, -1, 1) eq '/';
 
-     $options_p->{src_dir} = '' unless $options_p->{src_dir};
-     my $src_dir = $options_p->{src_dir};
+     $options->{src_dir} = '' unless $options->{src_dir};
+     my $src_dir = $options->{src_dir};
 
      #####
      # change to the source directory
      #
      my $restore_dir = cwd();
      chdir $src_dir if $src_dir;
-    
+ 
+     ########
+     # Add destination directory to the tar file
+     #
      my $contents;     
      if( $dest_dir ) {
          unless ($tar->taradd($dest_dir, '')) {
@@ -533,17 +556,28 @@ sub tar
          }
      }
 
-     my (%dirs, $dir_name, $file_dir, @file_dirs);
-     my ($file_name, $file_contents);
 
-     foreach $file_name (@files) {
+     ##########
+     # Make a separate copy of @files. Because changes in
+     # $file_name are reflected back into @files, if @files
+     # is @_, this would be reflected back into the calling
+     # @_.
+     #
+     my @files = @_;
+
+     ##############
+     # Keep track of the directories put in tar archive so
+     # do not duplicate any.
+     #
+     my %dirs = (); 
+     foreach my $file_name (@files) {
  
          #######
          # Add directory path to the archive file
          #
-         (undef, $file_dir) = File::Spec->splitpath( $file_name ) ;
-         @file_dirs = File::Spec->splitdir($file_dir);
-         $dir_name = $dest_dir;
+         (undef, my $file_dir) = File::Spec->splitpath( $file_name ) ;
+         my @file_dirs = File::Spec->splitdir($file_dir);
+         my $dir_name = $dest_dir;
          foreach $file_dir (@file_dirs) {
              $dir_name = File::Spec::Unix->catdir( $dir_name, $file_dir) if $dir_name;
              unless( $dirs{$dir_name} ) {
@@ -563,15 +597,15 @@ sub tar
              $file_name = File::Spec->rel2abs($file_name);
              warn "Cannot read contents of $file_name\n";
              chdir $restore_dir;
-             $tar->close( );
+             $tar->tarclose( );
              return undef;
          }
          binmode CONTENT;
-         $file_contents = join '',<CONTENT>;
+         my $file_contents = join '',<CONTENT>;
          close CONTENT;
 
          #######
-         # Add the file to the file archive
+         # Add the file to the tar archive file
          #  
          $file_name = File::AnySpec->os2fspec('Unix', $file_name);
          $file_name =  File::Spec::Unix->catfile($dest_dir,$file_name) if $dest_dir;
@@ -584,7 +618,7 @@ sub tar
      chdir $restore_dir;
 
      $tar->tarclose( );
-     return $options_p->{tar_file};
+     return $options->{tar_file};
 }
 
 
@@ -594,30 +628,41 @@ sub tar
 #
 sub untar
 {
-     return undef unless(defined($_[0]));      
-     shift @_ if $_[0] eq 'Archive::TarGzip';  # drop self on object call 
-
-     unless(@_) {
-         warn "No arguments.\n";
-         return undef;
-     }
+     ######
+     # This subroutine uses no object data.
+     #
+     shift @_ if UNIVERSAL::isa($_[0],__PACKAGE__);
 
      ######
-     # Check to see if first argument is an option
+     # pop last argument if last argument is an option
      #
-     my $args_p;
-     if( ref($_[0]) eq 'ARRAY') {
-          my %args = @{shift @_};
-          $args_p = \%args;
+     my $options  = {};
+     my $tar;  
+     if( ref($_[-1]) ) {
+
+         $options = pop @_ ;
+         if( ref($options) eq 'ARRAY') {
+              my %options = @{$options};
+              $options = \%options;
+         }
+         return undef unless $tar = new Archive::TarGzip($options);
+
+         #######
+         # Add any inputs files to the extract file hash
+         #
+         FileList2Sel(@_, $tar->{TarGzip}->{extract_files});
      }
-     elsif (ref($_[0]) eq 'HASH') {
-          $args_p = shift @_;
+
+     else {
+         my %options = @_;
+         $options = \%options;
+         return undef unless $tar = new Archive::TarGzip($options);
      }
 
      ######
      # Process options
      #
-     my $tar_file = $args_p->{tar_file};
+     my $tar_file = $options->{tar_file};
      unless( $tar_file ) {
           warn( "No tar file\n" );
           return undef;
@@ -634,24 +679,18 @@ sub untar
      ########
      # Attach to an existing tar file
      #
-     $args_p->{tar_flag} = '<';
-     my $tar;  
-     return undef unless $tar = new Archive::TarGzip($args_p);
+     $options->{tar_flag} = '<';
      return undef unless $tar->taropen();
 
-     #######
-     # Add any inputs files to the extract file hash
-     #
-     FileList2Sel(@_, $tar->{TarGzip}->{extract_files});
 
      ########
      # Change to the destination directory where place the 
      # extracted files.
      #
      my $restore_dir = cwd();
-     if ($args_p->{dest_dir}) {
-         mkpath($args_p->{dest_dir});
-         chdir $args_p->{dest_dir};
+     if ($options->{dest_dir}) {
+         mkpath($options->{dest_dir});
+         chdir $options->{dest_dir};
      }
 
      my ($tar_dir, $file_name, $dirs);
@@ -718,35 +757,67 @@ __END__
 
 =head1 NAME
 
-Archive::TarGzip - save and restore file to and from compressed tape archives (tar)
+Archive::TarGzip - save and restore files to and from compressed tape archives (tar)
 
 =head1 SYNOPSIS
 
+ ######
+ # Subroutine Interface
+ #  
+ use Archive::TarGzip qw(parse_header tar untar)
 
- use Archive::TarGzip [qw(parse_header tar untar)];
+ $tar_file = tar(@file, \%options)
+ $tar_file = tar(@file, \@options)
 
- $tar_file = Archive::TarGzip->tar(@file, [\%options or\@options]);
- $tar_file = tar(@file, [\%options or\@options]); # only if imported
+ $success = untar(@options)
+ $success = untar(\%options)
+ $success = untar(\@options)
+ $success = untar(@file, \%options)
+ $success = untar(@file, \@options)
 
- $success = Archive::TarGzip->untar([@file], \%options or\@options or @options);
- $success = untar([@file], \%options or\@options or @options); # only if imported
+ \%tar_header = parse_header($buffer)
 
+ ###### 
+ # Class Interface
+ #
+ use Archive::TarGzip
+ use vars qw(@ISA)
+ @ISA = qw(Archive::TarGzip)
 
- $tar = new Archive::TarGip( ); # best for Archive::Tar methods
- $tar = new Archive::TarGip( $filename or filehandle, [$compress]);
+ $tar_file = __PACkAGE__->tar(@file, \%options)
+ $tar_file = __PACkAGE__->tar(@file, \@options)
 
- $tar = new Archive::TarGip( \%options or\@options); # best for Archive::TarGzip methods
+ $success = __PACkAGE__->untar(@options)
+ $success = __PACkAGE__->untar(\%options)
+ $success = __PACkAGE__->untar(\@options)
+ $success = __PACkAGE__->untar(@file, \%options)
+ $success = __PACkAGE__->untar(@file, \@options)
 
- $tar_handle = $tar->taropen( $tar_file, $compress, [\%options or\@options]);
- $success = $tar->taradd($file_name, $file_contents) ;
- $success  = $tar->tarwrite();
- \%tar_header = $tar->tarread(@file, [\%options or\@options]);
- \%tar_header = $tar->tarread(\%options or\@options);
- $status = $tar->target( \$buffer, $size);
- $success = $tar->tarclose();
+ \%tar_header = __PACkAGE__->parse_header($buffer) 
 
- \%tar_header = Archive::TarGzip->parse_header($buffer) ;
- \%tar_header = parse_header($buffer);  # only if imported
+ ######
+ # Object Interface
+ # 
+ use Archive::TarGzip;
+
+ $tar = new Archive::TarGip( ) 
+ $tar = new Archive::TarGip( \%options )
+ $tar = new Archive::TarGip( \@options ) 
+ $tar = new Archive::TarGip( $filename or \*filehandle, \%options )
+ $tar = new Archive::TarGip( $filename or \*filehandle, \@options )
+ $tar = new Archive::TarGip( $filename or \*filehandle, $compress, @options)
+
+ $tar_handle = $tar->taropen( $tar_file, $compress, [\%options or\@options])
+ $success = $tar->taradd($file_name, $file_contents)
+ $success  = $tar->tarwrite()
+
+ \%tar_header = $tar->tarread(\%options)
+ \%tar_header = $tar->tarread(\@options)
+ \%tar_header = $tar->tarread(@file, \%options)
+ \%tar_header = $tar->tarread(@file, \@options)
+
+ $status = $tar->target( \$buffer, $size)
+ $success = $tar->tarclose()
 
 
 =head1 DESCRIPTION
@@ -823,10 +894,10 @@ extract files in this list.
 
 =item new method
 
- $tar = new Archive::TarGip( );
- $tar = new Archive::TarGip( $filename or filehandle, [$compress]);
+ $tar = new Archive::TarGzip( );
+ $tar = new Archive::TarGzip( $filename or filehandle, [$compress]);
 
- $tar = new Archive::TarGip( \%options or\@options);
+ $tar = new Archive::TarGzip( \%options or\@options);
 
 The new method creates a new tar object. 
 The Archive::TarGzip::new method is the only methods that hides
@@ -931,6 +1002,7 @@ The $tar_header keys are as follows:
  header_only
  skip_file
  data
+ file_position
 
 =item target method
 
@@ -1039,9 +1111,173 @@ follow on the next lines. For example,
  t/Data/Str2Num.pm
  t/Data/Str2Num.t'
 
+=head1 QUALITY ASSURANCE
+
+Running the test script 'Gzip.t' found in
+the "Archive-TarGzip-$VERSION.tar.gz" distribution file verifies
+the requirements for this module.
+
+All testing software and documentation
+stems from the 
+Software Test Description (L<STD|Docs::US_DOD::STD>)
+program module 't::Archive::TarGzip',
+found in the distribution file 
+"Archive-TarGzip-$VERSION.tar.gz". 
+
+The 't::Archive::TarGzip' L<STD|Docs::US_DOD::STD> POD contains
+a tracebility matix between the
+requirements established above for this module, and
+the test steps identified by a
+'ok' number from running the 'Gzip.t'
+test script.
+
+The t::Archive::TarGzip' L<STD|Docs::US_DOD::STD>
+program module '__DATA__' section contains the data 
+to perform the following:
+
+=over 4
+
+=item *
+
+to generate the test script 'Gzip.t'
+
+=item *
+
+generate the tailored 
+L<STD|Docs::US_DOD::STD> POD in
+the 't::Archive::TarGzip' module, 
+
+=item *
+
+generate the 'Gzip.d' demo script, 
+
+=item *
+
+replace the POD demonstration section
+herein with the demo script
+'Gzip.d' output, and
+
+=item *
+
+run the test script using Test::Harness
+with or without the verbose option,
+
+=back
+
+To perform all the above, prepare
+and run the automation software as 
+follows:
+
+=over 4
+
+=item *
+
+Install "Test_STDmaker-$VERSION.tar.gz"
+from one of the respositories only
+if it has not been installed:
+
+=over 4
+
+=item *
+
+http://www.softwarediamonds/packages/
+
+=item *
+
+http://www.perl.com/CPAN-local/authors/id/S/SO/SOFTDIA/
+
+=back
+  
+=item *
+
+manually place the script tmake.pl
+in "Test_STDmaker-$VERSION.tar.gz' in
+the site operating system executable 
+path only if it is not in the 
+executable path
+
+=item *
+
+place the 't::Archive::TarGzip' at the same
+level in the directory struture as the
+directory holding the 'Tie::Gzip'
+module
+
+=item *
+
+execute the following in any directory:
+
+ tmake -test_verbose -replace -run -pm=t::Archive::TarGzip
+
+=back
+
+
 
 =head1 NOTES
 
+=head2 FILES
+
+The installation of the
+"Archive-TarGzip-$VERSION.tar.gz" distribution file
+installs the 'Docs::Site_SVD::Archive_TarGzip'
+L<SVD|Docs::US_DOD::SVD> program module.
+
+The __DATA__ data section of the 
+'Docs::Site_SVD::Archive_TarGzip' contains all
+the necessary data to generate the POD
+section of 'Docs::Site_SVD::Archive_TarGzip' and
+the "Archive-TarGzip-$VERSION.tar.gz" distribution file.
+
+To make use of the 
+'Docs::Site_SVD::Archive_TarGzip'
+L<SVD|Docs::US_DOD::SVD> program module,
+perform the following:
+
+=over 4
+
+=item *
+
+install "ExtUtils-SVDmaker-$VERSION.tar.gz"
+from one of the respositories only
+if it has not been installed:
+
+=over 4
+
+=item *
+
+http://www.softwarediamonds/packages/
+
+=item *
+
+http://www.perl.com/CPAN-local/authors/id/S/SO/SOFTDIA/
+
+=back
+
+=item *
+
+manually place the script vmake.pl
+in "ExtUtils-SVDmaker-$VERSION.tar.gz' in
+the site operating system executable 
+path only if it is not in the 
+executable path
+
+=item *
+
+Make any appropriate changes to the
+__DATA__ section of the 'Docs::Site_SVD::Archive_TarGzip'
+module.
+For example, any changes to
+'Tie::Gzip' will impact the
+at least 'Changes' field.
+
+=item *
+
+Execute the following:
+
+ vmake readme_html all -pm=Docs::Site_SVD::Archive_TarGzip
+
+=back
+
 =head2 AUTHOR
 
 The holder of the copyright and maintainer is
